@@ -61,7 +61,8 @@ class Display:
         self.cells_theta = math.radians(0 - world.implementation.cell_transformation.rotation) + self.habitat_theta
         self.cells_size = world.implementation.cell_transformation.size / 2
         self.habitat_size = hsize
-
+        self.cell_polygons = []
+        self.habitat_polygon = None
         self._draw_cells__()
         plt.tight_layout()
 
@@ -69,8 +70,8 @@ class Display:
         [p.remove() for p in reversed(self.ax.patches)]
         for cell in self.world.cells:
             color = self.occlusion_color if cell.occluded else self.cell_color
-            self.ax.add_patch(RegularPolygon((cell.location.x, cell.location.y), self.world.configuration.cell_shape.sides, self.cells_size, facecolor=color, edgecolor=self.cell_edge_color, orientation=self.cells_theta, zorder=-2, linewidth=1))
-        self.ax.add_patch(RegularPolygon((self.xcenter, self.ycenter), self.world.implementation.space.shape.sides, self.habitat_size, facecolor=self.habitat_color, edgecolor=self.habitat_edge_color, orientation=self.habitat_theta, zorder=-3))
+            self.cell_polygons.append(self.ax.add_patch(RegularPolygon((cell.location.x, cell.location.y), self.world.configuration.cell_shape.sides, self.cells_size, facecolor=color, edgecolor=self.cell_edge_color, orientation=self.cells_theta, zorder=-2, linewidth=1)))
+        self.habitat_polygon = self.ax.add_patch(RegularPolygon((self.xcenter, self.ycenter), self.world.implementation.space.shape.sides, self.habitat_size, facecolor=self.habitat_color, edgecolor=self.habitat_edge_color, orientation=self.habitat_theta, zorder=-3))
 
     def set_occlusions(self, occlusions: Cell_group_builder):
         self.world.set_occlusions(occlusions)
@@ -79,24 +80,37 @@ class Display:
     def set_agent_marker(self, agent_name: str, marker: Path):
         self.agents_markers[agent_name] = marker
 
-    def add_trajectories(self, trajectories: Trajectories, colors={}, alpha: float = 0.5):
+    def add_trajectories(self, trajectories: Trajectories, colors={}, alphas={}):
         agents = trajectories.get_agent_names()
         for index, agent in enumerate(agents):
             locations = trajectories.get_agent_trajectory(agent).get("location")
             x = locations.get("x")
             y = locations.get("y")
             color = list(matplotlib.colors.cnames.keys())[index]
+            alpha = 0.5
             if agent in colors:
                 color = colors[agent]
+            if agent in alphas:
+                alpha = alphas[agent]
             for i in range(len(x)-1):
                 lcolor = None
+                lalpha = None
+                if type(alpha) is list:
+                    lalpha = alpha[i]
+                else:
+                    lalpha = alpha
                 if type(color) is numpy.ndarray:
                     lcolor = color[i]
                 else:
                     lcolor = color
-                self.ax.plot([x[i], x[i+1]], [y[i], y[i+1]], color=lcolor, alpha=alpha, linewidth=3)
+                self.ax.plot([x[i], x[i+1]], [y[i], y[i+1]], color=lcolor, alpha=lalpha, linewidth=3)
 
-    def cell(self, cell: Cell = None, cell_id: int = -1, coordinates: Coordinates = None, color="white"):
+
+    def cell(self, cell: Cell = None, cell_id: int = -1, coordinates: Coordinates = None, color=None, edge_color=None):
+        if color is None:
+            color = self.cell_color
+        if edge_color is None:
+            edge_color = self.cell_edge_color
         if cell is None:
             if cell_id == -1:
                 if coordinates is None:
@@ -109,16 +123,27 @@ class Display:
                     raise RuntimeError("cell coordinates not found")
             else:
                 cell = self.world.cells[cell_id]
-        self.ax.add_patch(RegularPolygon((cell.location.x, cell.location.y), self.world.configuration.cell_shape.sides, self.cells_size, facecolor=color, edgecolor=self.cell_edge_color, orientation=self.cells_theta, linewidth=1))
+        self.cell_polygons[cell.id].set_facecolor(color)
+        self.cell_polygons[cell.id].set_edgecolor(edge_color)
+
 
     def circle(self, location: Location, radius: float, color, alpha: float = 1.0):
         circle_patch = plt.Circle((location.x, location.y), radius, color=color, alpha=alpha)
         return self.ax.add_patch(circle_patch)
 
-    def arrow(self, beginning: Location, theta: float, dist: float, color, head_width: float = .02, alpha: float = 1.0):
-        ending = beginning.copy().move(theta=theta, dist=dist)
+    def arrow(self, beginning: Location, ending: Location = None, theta: float = 0, dist: float = 0, color="black", head_width: float = .02, alpha: float = 1.0, existing_arrow: matplotlib.patches.FancyArrowPatch = None) -> matplotlib.patches.FancyArrowPatch:
+        if ending is None:
+            ending = beginning.copy().move(theta=theta, dist=dist)
         length = ending - beginning
-        return self.ax.arrow(beginning.x, beginning.y, length.x, length.y, color=color, head_width=head_width, length_includes_head=True, alpha=alpha)
+        if existing_arrow:
+            existing_arrow.set_data(x=beginning.x, y=beginning.y, dx=length.x, dy=length.y, head_width=head_width)
+            existing_arrow.set_color(color)
+            existing_arrow.set_alpha(alpha)
+            return existing_arrow
+        else:
+            new_arrow = self.ax.arrow(beginning.x, beginning.y, length.x, length.y, color=color, head_width=head_width, length_includes_head=True, alpha=alpha)
+            new_arrow.animated = self.animated
+            return new_arrow
 
     def agent(self, step: Step = None, agent_name: str = None, location: Location = None, rotation: float = None, color=None, size: float = 40.0, show_trajectory: bool = True, marker: Path=None):
         if step:
